% we want to 1) show a joint histogram of linear responses with no
% conditional variance dependencies, and obtain from its normalized responses a
% joint-product target C 2) show a linear joint histogram with a
% conditional variance dependence introduced and its normalized responses,
% which are correlated 3) show (2) after adaptation relative to C

function show_adj_cond_var
    N = 40;
    k = ones(N)/N*10;
    s = 0.2;
    N_samples = 10000;
    samples1 = randn(N, N_samples);
    
    hist = true;
    
    fs = 12;
    ms = 1;
    
    
    
    bins1N = {0:0.1:2, 0:0.1:2};
    samples1N = normalize(samples1, k, s); 
    
   
    
    %introduce a dependence
    epsilon = 0.75;
    samples2 = samples1;
    
    x = samples2(1,:);
    y = samples2(2,:);

    var_pred = 1 - epsilon* var(x) + epsilon  * x.^2;
    y_adj = y .* (var_pred).^0.5;


    before = var(y);
    after = var(y_adj);
    samples2(1,:) = x;
    samples2(2,:) = y_adj;
    
    samples2N = normalize(samples2, k, s);
    
    samples3N = adaptation(samples1, samples2, k, s);
    
    subplot(3,3,1)
    
    
    if hist
        bins1 = {-5:0.1:5, -5:0.1:5};
        cnt = hist3(samples1(1:2, :)', bins1);
        cnt = log(1+flipdim(cnt, 1))';
        imagesc(bins1{1}, bins1{2}, cnt); colormap(gray); axis square;
     else
        h=plot(samples1(1,:), samples1(2,:), 'o');
        set(h, 'Color',[0,0,0])
        set(h,'MarkerSize',ms)
        set(h,'MarkerFaceColor',[0,0,0])


    end
    h=xlabel('R_1');
    set(h, 'FontSize', fs);
    h=ylabel('R_2');
    set(h, 'FontSize', fs);
    axis([-5,5,-5,5])
    set(gca, 'XTick',[-5 0 5])
    set(gca, 'YTick',[-5 0 5])
    h=title('A');
    set(h,'FontSize',18)
    axis square;
    %scoot over
    h=subplot(3,3,1);
    pos = get(h,'Position');
    pos(1) = pos(1) + 0.05;
    set(h, 'Position',pos);
    
    h=subplot(3,3,2);
    pos = get(h,'Position');
    set(h,'Position',pos);
    if hist
        cnt = hist3(samples2(1:2, :)', bins1);
        cnt = log(1+flipdim(cnt, 1))';
        imagesc(bins1{1}, bins1{2}, cnt).^0.5; colormap(gray); axis square;
    else
        h=plot(samples2(1,:), samples2(2,:), 'o');
        set(h, 'Color',[0,0,0])
        set(h,'MarkerSize',ms)
        set(h,'MarkerFaceColor',[0,0,0])
        axis square;

    end
    axis([-5,5,-5,5])
    set(gca, 'XTick',[-5 0 5])
    set(gca, 'YTick',[-5 0 5])
    h=xlabel('R_1');
    set(h, 'FontSize', fs);
    h=ylabel('R_2');
    set(h, 'FontSize', fs);
    h=title('B');
    set(h,'FontSize',18)
    axis square;
    %scoot
    h=subplot(3,3,2);
    pos = get(h,'Position');
    pos(1) = pos(1) + 0.02;
    set(h, 'Position',pos);
    
    subplot(3,3,3)
    test_gauss_cv2
     h=title('C');
    set(h,'FontSize',18)
    
    %measure conditional variances
    
    [xm1, yv1] = measureCondVar(samples1(1,:).^2, samples1(2,:), 10);
    [xm2, yv2] = measureCondVar(samples2(1,:).^2, samples2(2,:), 10);
    
    [xm1N, yv1N] = measureCondVar(samples1N(1,:).^2, samples1N(2,:), 10);
    [xm2N, yv2N] = measureCondVar(samples2N(1,:).^2, samples2N(2,:), 10);
    [xm3N, yv3N] = measureCondVar(samples3N(1,:).^2, samples3N(2,:), 10);

    h=subplot(3,3,5);
    pos = get(h,'Position');
    pos(1) = pos(1) - 0.15;
    pos(2) = pos(2) - 0.02;
    pos(4) = pos(4) * 0.75;
    set(h, 'Position',pos);

    h=plot(xm1, yv1,'Color',[0.5, 0.5, 0.5]);
    set(h,'LineWidth',2)

    hold on
    h=plot(xm2, yv2,'k');
    set(h,'LineWidth',2)
    axis([0, max(xm1), 0, 3])
    h=xlabel({'R_1^2'});
    set(h,'FontSize',fs)
    p = get(h, 'Position');
    p(2) = p(2) - 0.01;
    set(h, 'Position',p)
    h=ylabel({'var(R_2 | R_1^2)'});
   
    h=title('D');
    set(h,'FontSize',18)
    
        
    h=subplot(3,3,6);
    pos = get(h,'Position');
    pos(1) = pos(1) - 0.075;
    pos(2) = pos(2) - 0.02;
    pos(4) = pos(4) * 0.75;
    set(h, 'Position',pos);
    
    h=plot(xm1N, yv1N,'Color',[0.5, 0.5, 0.5]);
    set(h,'LineWidth',2)
    hold on
    h=plot(xm2N, yv2N,'k');
    set(h,'LineWidth',2)
    h=plot(xm3N, yv3N,'--k');
    set(h,'LineWidth',2)
    
    h=xlabel({'N_1^2'});
    set(h,'FontSize',fs)
    p = get(h, 'Position');
    p(2) = p(2) - 0.01;
    set(h, 'Position',p);

    h=ylabel({'var(N_2 | N_1^2)'});
    set(h,'FontSize',fs)
    
    
     h=title('E');
    set(h,'FontSize',18)
    axis([0, max(xm2N), 0, 0.1])
    set(h,'LineWidth',2)
    

    
    
end

function test_csf(samples, k, sigma)
    samples(1,:) = linspace(0, 5, size(samples, 2));
    samplesN = normalize(samples, k,  sigma);
    plot(samples(1,:), samplesN(1,:))
end

%s: feedforward responses
%k: weight matrix
%sigma: semisaturation constant
function normed = normalize(s, k, sigma)
    normed = s.^2 ./ (sigma + k*(s.^2));
end


function samples2N = adaptation(samples1, samples2, k, s)
    samples1N = normalize(samples1, k, s);
    C = (samples1N * samples1N') / size(samples1N,2);
    temp = 0.1;
    k2 = k;
    for iter=1:2000
        samples2N = normalize(samples2, k2, s);
        C2 = (samples2N * samples2N') / size(samples2N,2);
        k2 = k2 + temp * (C2 - C);
      
        %imagesc(C2); drawnow
    end
end
