clear all



%figure 1 plots various cartoon effects
%first make the two cartoon plots
s = get_s(7,360,30);
sv = 0.3;
sm = 0.4;
suppression = @(ori) 1-sm*exp(-1 * (ori - 0).^2/(2*sv^2));
sup_n = suppression(linspace(-1, 1, size(s,1)));
sup_s = suppression(linspace(-1, 1, size(s,2)));

ns = diag(sup_n) * s;
ss = s * diag(sup_s);
fs = 12;
lw = 2;

subplot(3,3,1)
h=plot(linspace(-90, 90, size(s,2)), ns','k');
set(h, 'LineWidth',2);
 
h=ylabel('Neural response');
set(h, 'FontSize',fs)
axis([-91, 91, 0, 1.05])
hold on
h=plot(linspace(-90, 90, size(s,2)), s(5,:)','k--');
set(h, 'LineWidth',lw);

h=plot(linspace(-90, 90, size(s,2)), ns(4,:)','r');
set(h, 'LineWidth',lw);

hold off

set(gca, 'FontSize',fs)
set(gca, 'XTick',[-90, -45, 0, 45, 90])
set(gca, 'YTick',[0.0, 0.5, 1])
h=title('A');
p = get(h, 'position');
set(h, 'position',[0,1.05 ,1.0])
set(h, 'FontSize', 20)


subplot(3,3,4)
h=plot(linspace(-90, 90, size(s,2)), ss','k');

set(h, 'LineWidth',2);
h=xlabel('Stimulus orientation');
set(h, 'FontSize',fs)
h=ylabel('Neural response');
set(h, 'FontSize',fs)
axis([-91, 91, 0, 1.05])
set(gca, 'XTick',[-90, -45, 0, 45, 90])
set(gca, 'YTick',[0.0, 0.5, 1])
hold on
h=plot(linspace(-90, 90, size(s,2)), s(5,:)','k--');
set(h, 'LineWidth',lw);

h=plot(linspace(-90, 90, size(s,2)), ss(4,:)','r');

set(h, 'LineWidth',lw);
hold off
h=title('D');
set(h, 'FontSize', 20)
p = get(h, 'position');
set(h, 'position',[0,1.05 ,1.0])
set(gca, 'FontSize',fs)

%now show environments and their corresponding joint-product distributions
pu = ones(1,11);

lw = 2;
sc=1.0;%subplot scaling
pu = pu / sum(pu);

pb = ones(1,11);
pb(6) = 5;
pb = pb / sum(pb);
sigma = get_sigma(30);
N = 121;
[kb4u,tb4u, tbu4b]=jnt_descend_t2(N,sigma,pu,0.0,0.01,0.03,1.0);
[kb4b,tb4b, tbb4b]=jnt_descend_t2(N,sigma,pb,0.0,0.01,0.03,1.0);

cfake = get_joint(tb4u, pu);
bfake = get_joint(tb4b, pb);
ubfake = get_joint(tb4u, pb);

breal = get_cov_real(tb4b, pb);
ubreal = get_cov_real(tb4u, pb);
creal = get_cov_real(tb4u, pu);
M=11;
oris_extra = linspace(0,pi,M+1);
oris = oris_extra(1:M);
center = ceil(M/2);
ori_center = oris(center);
oris = oris - ori_center + pi/2;
oris = oris / (2*pi) * 360 - 90;

subplot(3,3,2);

bar(oris, pu,1.0,'FaceColor',[0.5,0.5,0.5]);
h=title('B');
set(h, 'FontSize', 20);


h=ylabel('Frequency');
set(h, 'FontSize',fs);
axis([-90 90 0 0.4])
set(gca, 'XTick',[-90, 0, 90])

subplot(3,3,5);
bar(oris, pb,1.0,'FaceColor',[0.5,0.5,0.5]);

h=title('E');
set(h, 'FontSize', 20);

h=xlabel('Stimulus orientation');
set(h, 'FontSize',fs)
h=ylabel('Frequency');
set(h, 'FontSize',fs);
axis([-90 90 0 0.4])
set(gca, 'XTick',[-90, 0, 90])

%now put the covariances
%scale everything together!
umin = min([min(creal(:)), min(ubreal(:)), min(breal(:))]);

umax = max([max(creal(:)-umin), max(ubreal(:)-umin), max(breal(:)-umin)]);


fmin = min([min(cfake(:)), min(ubfake(:)), min(bfake(:))]);

fmax = max([max(cfake(:)-fmin), max(ubfake(:)-fmin), max(bfake(:)-fmin)]);


creal_scaled = (creal-umin)/umax;

%rescale them
breal = (get_cov_real(tb4b, pb) - umin)/umax;
ubreal = (get_cov_real(tb4u,pb) - umin)/umax;
imgscale = 64;

subplot(3,3,3)
image(linspace(-90,90,N), linspace(-90, 90, N), flipdim(creal_scaled*imgscale,2));
axis square;
set(gca, 'YTick',[-90, 0, 90])
set(gca, 'XTick',[-90, 0, 90])
set(gca, 'YTickLabel',-1*[-90, 0, 90])

h=ylabel('Preferred orientation');
set(h, 'FontSize',fs)
set(gca, 'FontSize',fs);
colormap('hot')
h=title('C');
set(h, 'FontSize', 20);

subplot(3,3,6);
image(linspace(-90,90,N), linspace(-90, 90, N),flipdim(ubreal*imgscale,2)); 
axis square;
set(gca, 'YTick',[-90, 0, 90])
set(gca, 'XTick',[-90, 0, 90])
set(gca, 'YTickLabel',-1*[-90, 0, 90])
h=xlabel('Preferred orientation');
set(h, 'FontSize',fs)
h=ylabel('Preferred orientation');
set(h, 'FontSize',fs)
set(gca, 'FontSize',fs);
h=title('F');
set(h, 'FontSize', 20);

for i=1:6
    h=subplot(3,3,i);
    pos = get(h, 'Position');
    pos(3:4) = pos(3:4) * 0.8;
    
    if i == 2 || i == 5
        pos(1) = pos(1) + 0.03;
    end
    pos(2) = pos(2) - 0.05;
    set(h, 'Position',pos);
end

%print(gcf, '-depsc2', 'fig1N')
